/*
*  Copyright 2022 by the author. It is protected by applicable copyright laws. This program does not
*  give you or anybody else a right or licence to distribute, modify, copy, compile, create derivatives.
*/

#include "Clause.h"
#include "Term.h"
#include "Literal.h"
#include "SortRule.h"
#include "dataTypeDef.h"
#include "HashTree.h"
#include "globalFunc.h"
#include "FileOp.h"

extern set<HashNode*, HashNodeCmpByCode> g_HashTree;
Clause::Clause() {
}

Clause::Clause(const Clause& orig) {
}

Clause::~Clause() {
}

Clause::Clause(vector<string>& vectsLit, UINT32 _claId): uClaId(_claId), Priority(0), uLitNum(0), uVarCount(0), uFuncDepth(0), uReuseTimes(0),adeductNum(0),pdeductNum(0),
        isSDeduct(false),reduntTimes(0),isUse(false),isAxtom(false),isGoal(false),isPureCla(false),clast(noState),noUnitDeductTimes(0),demodulator(false),orgCla(false),goaldeepth(litWeight),
        deduceDeepth(0),copyFrom(nullptr),copyTo(nullptr),topCopyCla(nullptr),noPairResFiFg(false),maxPred(0),minPred(0),maxPairPred(0),minPairPred(0),claAttr(0),termCount12(0),termCount11(0),termCount21(0){
    LitPtr = new Literal*[vectsLit.size()]; 
    for (uLitNum = 0; uLitNum < vectsLit.size(); ++uLitNum) {
        string& _strLit = vectsLit[uLitNum]; 
        TrimStr(_strLit); 
        char* ch = const_cast<char*> (_strLit.c_str()); 		  
        Term* litSubT = GenerateTerm(ch);
        if (uFuncDepth < litSubT->uLevel) uFuncDepth = litSubT->uLevel; 
        uVarCount += litSubT->uVarCount;		
    }
}

UINT32 Clause::FstTermCode(char* &ch) {
    string sTermCode = "";
    INT32 iPredCode;
    bool isNav = false; 
    if (*ch == '~') {
        isNav = true;
        ++ch;
    }
    bool isFunc = false, isEqn = false; 
    if ('f' == *ch)
        isFunc = true;
    else if ('E' == *ch)
        isEqn = true;
    while (*(++ch) != '(') {
        sTermCode += *ch;
    }
    iPredCode = sTermCode == "" ? 0 : stoi(sTermCode); 
    if (isFunc)
        Set1stTProp(iPredCode, TFunc);
    else if (isEqn) {
        if (isNav)
            Set1stTProp(iPredCode, TNegEqn); 
        else
            Set1stTProp(iPredCode, TEqn); 
    } else {
        if (isNav)
            Set1stTProp(iPredCode, TNegPred); 
        else
            Set1stTProp(iPredCode, TPred); 
    }
    return iPredCode;
}

Term* Clause::GenerateTerm(char* &sLit) {
    string sTermCode;
    Term** vSubTerms = new Term*[255];
    Term* term = new Term(FstTermCode(sLit)); 
    ++sLit;
    while (1) {
        sTermCode = "";
        if ('x' == *sLit) {
            ++sLit;
            while (*sLit != ',' && *sLit != ')') {
                sTermCode += *sLit; 
                ++sLit;
            }
            UINT32 oldVarCode = stoi(sTermCode); 
            UINT16 uTmpMaxVarId; 
            vSubTerms[term->uSubTermNum]->uVarCount = 1;
            term->uSubTermNum++;
            if (uTmpMaxVarId > 0)
                term->uMaxVarId = uTmpMaxVarId;
            ++term->uVarCount;
        } else if ('f' == *sLit) {
            Term *subT = GenerateTerm(sLit);
            if(term->isFuncTerm())
                term->uLevel = subT->uLevel + 1; 
            else if(term->isPredTerm()){
                if(term->uLevel < subT->uLevel + 1) 
                    term->uLevel = subT->uLevel + 1;
            }
            term->fSteadyValue += subT->fSteadyValue * 0.5f; 
            term->uVarCount += subT->uVarCount; 
            if (term->uMaxVarId < subT->uMaxVarId)
                term->uMaxVarId = subT->uMaxVarId;
        } else if ('a' == *sLit) {
            ++sLit;
            while (*sLit != ',' && *sLit != ')') {
                sTermCode += *sLit;
                ++sLit;
            }
            UINT32 code = stoi(sTermCode); 
            code = EncodeCon(code); 
            Term* baseT = new Term(code);	         
            ++(term->fSteadyValue); 
        }
        if (*sLit == ')') { 
            if(term->isFuncTerm())
                term->fSteadyValue /= term->uSubTermNum; 			
           else if(term->isPredTerm())
               term->fSteadyValue = (term->fSteadyValue + term->uLevel)/term->uSubTermNum; 
            ++sLit;
            break;
        }
        ++sLit;
    }
    delete[]vSubTerms;
    return term;
}

void Clause::DelLits(bool isDelPred) {
    for (UINT16 i = 0; i < uLitNum; ++i) {
        if (isDelPred)LitPtr[i]->DelPred();
        delete(LitPtr[i]);
    }
    delete[]LitPtr;
    LitPtr = nullptr;
}

void Clause::DelOneLit(Literal* lit, bool isDelPred) {
    for (UINT16 i = 0; i < uLitNum; ++i) {
        if (LitPtr[i] == lit) {
            if (isDelPred)LitPtr[i]->DelPred();
            delete(LitPtr[i]);
        }
    }
}

void Clause::toString(string&sCla) {
    UINT16 i = 0;
    sCla = "[" + to_string(uClaId) + "]";
    while (1) {
        sCla += LitPtr[i]->subTerm->ToString();
        ++i;
        if (i < uLitNum)
            sCla += "+";
        else
            break;
    }
}

void Clause::addRForCompetiton(){
    if(uLitNum > 1){
        return;
    }
    string sCla;
    toPureString(sCla);
    FileOp::prinfResForE(sCla);
}

void Clause::toPureString(string&sCla) {
    UINT16 i = 0;
    sCla = "[" + to_string(uClaId) + "]";
    while (1) {
        sCla += LitPtr[i]->subTerm->ToString();
        ++i;
        if (i < uLitNum)
            sCla += "+";
        else
            break;
    }
}

void Clause::Print() {
    UINT16 i = 0;
    while (1) {
        cout << "[" << (i + 1) << "]";
        LitPtr[i]->subTerm->Print();
        ++i;
        if (i < uLitNum)
            cout << "+";
        else
            break;
    }
    cout << endl;
}

int Clause::compFuncDeepthCla() {
    UINT16 i = 0;
    int maxLitCount = 0;
    int count;
    while (1) {
        count = 0;
        LitPtr[i]->subTerm->compFuncDeepth2(count);
        if(count>maxLitCount)
            maxLitCount = count;
        ++i;
        if (i < uLitNum)
            ;
        else
            break;
    }
    return maxLitCount;
}

bool Clause::HasTermBind() {
    UINT16 i = 0;
    while (1) {
        if(LitPtr[i]->subTerm->IsExistBind()){
            return true;
        }
        ++i;
        if (i < uLitNum)
            continue;
        else
            return false;
    }
}

void Clause::SortLit() {
    if (1 == uLitNum) return;
    sort(LitPtr, LitPtr + uLitNum, SortRule::LitCmp);
    for (UINT16 uCol = 0; uCol < uLitNum; ++uCol) 
    {
        LitPtr[uCol]->ReCol(uCol + 1);
    }
}

void Clause::SortLitDuct() {
    if (1 == uLitNum) return;
    sort(LitPtr, LitPtr + uLitNum, SortRule::LitCmpDuct);
    for (UINT16 uCol = 0; uCol < uLitNum; ++uCol) 
    {
        LitPtr[uCol]->ReCol(uCol + 1);
    }
}

UINT32 Clause::countClaTermE(UINT8 funCoff, UINT8 varCoff){
    UINT32 litTermCount;
    UINT32 claTermCount = 0;
    for(int i = 0; i<uLitNum; ++i){
        litTermCount = 0;
        LitPtr[i]->subTerm->countTermE(litTermCount,funCoff,varCoff);
        claTermCount += litTermCount;
    }
    return claTermCount;
}

void Clause::initOrgClaAttr()
{
    UINT32 maxSize = 0, minSize = 12000;
    UINT32 maxPairSize = 0, minPairSize = 12000;
    UINT32 count = 0;
    vector <UINT8> saveTmp;
    saveTmp.reserve(6);
    for (UINT16 i = 0; i < uLitNum; ++i) {
        if (LitPtr[i]->isConLit()) {
            LitPtr[i]->litAttr = 0; 
            saveTmp.push_back(0);
        } else if (LitPtr[i]->isSingleVarliteral() && !(LitPtr[i]->isShareVarLiteral())) {
            LitPtr[i]->litAttr = 1; 
            saveTmp.push_back(1);
        } else if (LitPtr[i]->isShareVarLiteral()) {
            LitPtr[i]->litAttr = 2; 
            saveTmp.push_back(2);
        }
        LitPtr[i]->countPred = LitPtr[i]->getPredLst()->size() - 1;
        LitPtr[i]->countPairPred = LitPtr[i]->getPairPredLst()->size();
        count = 0;
        LitPtr[i]->subTerm->countTermE(count, 1, 2);
        LitPtr[i]->termCount12 = count;
        termCount12 += count;
        count = 0;
        LitPtr[i]->subTerm->countTermE(count, 1, 1);
        LitPtr[i]->termCount11 = count;
        termCount11 += count;
        count = 0;
        LitPtr[i]->subTerm->countTermE(count, 2, 1);
        LitPtr[i]->termCount21 = count;
        termCount21 += count;
        if (maxSize < LitPtr[i]->countPred) {
            maxSize = LitPtr[i]->countPred;
        } else if (minSize > LitPtr[i]->countPred) {
            minSize = LitPtr[i]->countPred;
        }
        if (maxPairSize < LitPtr[i]->countPairPred) {
            maxPairSize = LitPtr[i]->countPairPred;
        } else if (minPairSize > LitPtr[i]->countPairPred) {
            minPairSize = LitPtr[i]->countPairPred;
        }
    }
    if (find(saveTmp.begin(), saveTmp.end(), 2) != saveTmp.end()) {
        claAttr = 2;
    } else if (find(saveTmp.begin(), saveTmp.end(), 1) != saveTmp.end()) {
        claAttr = 1;
    } else
        claAttr = 0; 
    saveTmp.clear();
    vector <UINT8>().swap(saveTmp);
    maxPred = maxSize;
    minPred = minSize;
    maxPairPred = maxPairSize;
    minPairPred = minPairSize;   
}

void Clause::exEqnTerm(){
    if((uLitNum != 1) || ((GetCodeProp(LitPtr[0]->subTerm->iTermCode) !=TEqn) && (uLitNum == 1)))
        return;
    else{
        bool change = false;
        Literal* lit = LitPtr[0];
        UINT32 countLeft = 0;
        UINT32 countRight = 0;
        Term* left = lit->subTerm->subTermPtr[0];
        Term* right = lit->subTerm->subTermPtr[1];
        left->countTerm(countLeft);
        right->countTerm(countRight);
        if (countLeft > countRight) {
            return;
        } 
        else if (countLeft == countRight) {
            if(left->isVar() && right->isVar()){
                if((left->iTermCode & 0xFF) > (right->iTermCode & 0xFF)){
                    return;
                }
                else 
                    change = true;
            }
            else if(left->isVar() && right->isConTerm()){
                return;
            }
            else if(left->isConTerm() && right->isVar()){
                change = true;
            }
            else if(left->isConTerm() && right->isConTerm()){
                if((left->iTermCode & 0xFF) > (right->iTermCode & 0xFF)){
                    return;
                }
                else 
                    change = true;
            }
        } 
        else
            change = true;
        if(change)
        {
            Term* tmp = nullptr;
            tmp = lit->subTerm->subTermPtr[0];
            lit->subTerm->subTermPtr[0] = lit->subTerm->subTermPtr[1];
            lit->subTerm->subTermPtr[1] = tmp;
        }
        return;
    }
}

void Clause::directEqn() {
    if (uLitNum > 1)
        return;
    for (int m = 0; m < uLitNum; m++) {
        if (LitPtr[m]->subTerm->isPosEqnTerm() || LitPtr[m]->subTerm->isNegEqnTerm()) {
            LitPtr[m]->exEqnTerm();
        }
    }
}
