/*
*  Copyright 2021 by the author. It is protected by applicable copyright laws. This program does not
*  give you or anybody else a right or licence to distribute, modify, copy, compile, create derivatives.
*/

#ifndef TERM_H
#define TERM_H
#include "dataTypeDef.h"
#include "globalFunc.h"
#include <stdlib.h>

class Term {
public:
    INT32 iTermCode;
    Term** subTermPtr;
    Term* vBinding; 	 
    float fSteadyValue; 
    UINT8 uSubTermNum; 
    UINT8 uLevel;
    UINT8 uMaxVarId; 
    UINT16 uVarCount; 
    termState termSt;
    UINT32 groundTermCount;
    Term(): iTermCode(0), subTermPtr(nullptr), vBinding(nullptr), fSteadyValue(0.0f), uSubTermNum(0), uLevel(0), uMaxVarId(0), uVarCount(0),termSt(noSt),groundTermCount(0){
    }
    Term(INT32 _iTermCode);
    virtual ~Term();
    inline bool isVar(){
        if(iTermCode<0)
            return true;
        else
            return false;
    }
    inline bool isConTerm(){
        if((uSubTermNum == 0) && (uVarCount == 0))
            return true;
        else
            return false;
    }
    inline float RelationValue() {
        return uMaxVarId / (uVarCount * 0.1f);
    }
    bool EqualInSameCla(const Term* t);
    bool EqualNoSameCla(const Term* t);
    inline bool isEqnTerm() {
        UINT32 uCodeProp = GetCodeProp(iTermCode);
        return (TEqn == uCodeProp || TNegEqn == uCodeProp);
    }
    inline bool isPLitTerm(){
         UINT32 uCodeProp = GetCodeProp(iTermCode);
        return (TPred == uCodeProp || TNegPred == uCodeProp);
    }
    inline bool isNegLit(){
        UINT32 uCodeProp = GetCodeProp(iTermCode);
        return (TNegEqn == uCodeProp || TNegPred == uCodeProp);
    }
     inline bool isPosEqnTerm() {
        UINT32 uCodeProp = GetCodeProp(iTermCode);
        return (TEqn == uCodeProp);
    }
     inline bool isNegEqnTerm() {
        UINT32 uCodeProp = GetCodeProp(iTermCode);
        return (TNegEqn == uCodeProp);
    }
     inline bool isFuncTerm() {
        UINT32 uCodeProp = GetCodeProp(iTermCode);
        return (TFunc == uCodeProp);
    }
      inline bool isPredTerm() {
        UINT32 uCodeProp = GetCodeProp(iTermCode);
        return ((TEqn == uCodeProp) ||(TNegEqn == uCodeProp) ||(TPred == uCodeProp) ||(TNegPred == uCodeProp) );
    }
    inline Term* GetBindingT() {
        Term* t = this;
        while (t->vBinding)
            t = t->vBinding;
        return t;
    }
    void Print();
    void compFuncDeepth(int& count);
    string ToString();
    string ToStringBind();
    bool IsExistBind();
    bool IsExistNoVarBind();
    bool IsGroundLit();
    bool IsSingleVarLit();
    bool IsShareVarLit();
    void countTerm(UINT32& TermCount);
    void countTermE(UINT32& TermCount,UINT8 funCoff, UINT8 varCoff);
    void countVarTerm(UINT32& TermCount);
    void recordTerm(vector<Term*>* termVec);
    bool similarTerm(Term* term);
    void termNum(int* number);
    bool suitSuper();
    void countUqVarTerm(UINT32& TermCount, vector<INT32>& save);
    bool hasVarOrBindVar();
};

class VarTermCmp : greater<Term*> {
public:
    bool operator()(const Term* t1, const Term* t2) const {
        return t1->iTermCode < t2->iTermCode;
    }
};

#endif /* TERM_H */

