/*
*  Copyright 2022 by the author. It is protected by applicable copyright laws. This program does not
*  give you or anybody else a right or licence to distribute, modify, copy, compile, create derivatives.
*/

#include <cstdlib>
#include "FileOp.h"
#include "InitFunction.h"
#include "cprover.h"
#include "Formula.h"
#include "Prover.h"
#include "dataTypeDef.h"
 #include "globalFunc.h"
#include "SocketOp.h"
using namespace std;
class funcAddGreater{
public:
    bool operator()(const funcAdd* term1, const funcAdd* term2)
    {
        return term1->func < term2->func;
    }
};
class literalAddGreater{
public:
    bool operator()(const literalAdd* term1, const literalAdd* term2)
    {
        return term1->literal < term2->literal;
    }
};

void findFuncEquality(char* &sym,vector<funcAdd*> &funcAddEquality){
    funcAdd *funcSym = new funcAdd();
    funcSym->func = "";
    funcSym->suffix = 1;
    int iBracketNum = 0;
    while (*sym != '(') {
        funcSym->func += *sym;
        ++sym;
    }
    ++iBracketNum;
    while (iBracketNum > 0) {
        if (*sym == 'f') {
            findFuncEquality(sym, funcAddEquality);
        }
        else if (*sym == ',') {
            ++(funcSym->suffix); 
        }
        else if (*sym == ')') {
            --iBracketNum;
            if (iBracketNum == 0)
                break;
        }
        if(iBracketNum == 0){
            break;
        }
        ++sym;
    }
    funcAddEquality.push_back(funcSym);
}

void findEquality(char* &sym,vector<funcAdd*> &funcAddEquality,vector<literalAdd*> &LiteralAddEquality){
    while(1){
        if(*sym == 'p'){
            literalAdd *literalSym = new literalAdd();
            literalSym->literal = "";
            literalSym->suffix = 1;
            int iBracketNum = 0;
            bool proposition = false;
            bool runOneTime = true;
            while(*sym != '('){
                if(*sym == ' '){
                    ++sym;
                    continue;
                }
                if((*sym == '|') || (*sym == ')')){
                    literalSym->suffix = 1;
                    proposition = true;
                    break;
                }
                literalSym->literal += *sym; 
                ++sym;
                if(((*sym< '0')||(*sym >'9')) && runOneTime){
                    proposition = true;
                    runOneTime = false;
                    break;
                }
                else if(((*sym >= '0') && (*sym <= '9'))&& runOneTime){
                    runOneTime = false;
                }
                
            }
            if(proposition){
                LiteralAddEquality.push_back(literalSym);
                continue;
            }
            ++iBracketNum;
            while(iBracketNum > 0){
                if(*sym == 'f'){
                    findFuncEquality(sym,funcAddEquality);
                }
                else if(*sym == ','){
                    ++(literalSym->suffix);
                }
                else if(*sym == ')'){
                    --iBracketNum;
                    if(iBracketNum == 0)
                        break;
                }
                ++sym;
            }
            LiteralAddEquality.push_back(literalSym);
        }
        if(*sym == 'f'){
            findFuncEquality(sym,funcAddEquality);
        }
        if(*sym == '.')
            break;
        ++sym;
    }
}

string getTerm(int suffix, int size, int timeFlag){
    string res = "";
    vector<char> leftTerm;
    vector<char> rightTerm;
    for(int i=0; i<= size; ++i){
        if(i == 1)
            continue;
        leftTerm.push_back(i+'A');
    }
    for(int i=0; i<= size; ++i){
        if(i == 0)
            continue;
        rightTerm.push_back(i+'A');
    }
    for(int i=0; i<suffix; ++i){
        swap(leftTerm[i],leftTerm[i+1]);
        swap(rightTerm[i],rightTerm[i+1]);
    }
    if(timeFlag == 0){
        if(leftTerm.size()!=0){
            res += "(";
            for(int i=0;i<leftTerm.size();++i){
                res += leftTerm[i];
                if(i<leftTerm.size()-1){
                    res += ",";
                }
            }
            res += ")";
        }
        vector<char>().swap(leftTerm);
        vector<char>().swap(rightTerm);
        return res;
    }
    else if(timeFlag == 1){
      if(rightTerm.size()!=0){
            res += "(";
            for(int i=0;i<rightTerm.size();++i){
                res += rightTerm[i];
                if(i<rightTerm.size()-1){
                    res += ",";
                }
            }
            res += ")";
        }
        vector<char>().swap(leftTerm);
        vector<char>().swap(rightTerm);
        return res;  
    }
    vector<char>().swap(leftTerm);
    vector<char>().swap(rightTerm);
    return res;  
}

void addFunc(ofstream &ofInfoStream,funcAdd* funcSym){
    for(int i=0; i< funcSym->suffix; ++i){
        ofInfoStream << "cnf(" + funcSym->func + "_substitution_" + to_string(i+1) +",axiom," << endl;
        ofInfoStream << "    ( A != B | " + funcSym->func +getTerm(i,funcSym->suffix,0)+"= " + funcSym->func + getTerm(i,funcSym->suffix,1)+" ))." << endl;
        ofInfoStream << endl;
    }
}

void addLiteral(ofstream &ofInfoStream,literalAdd* literalSym){
    for(int i=0; i< literalSym->suffix; ++i){
        ofInfoStream << "cnf(" + literalSym->literal + "_substitution_" + to_string(i+1) +",axiom," << endl;
        ofInfoStream << "    ( A != B | ~ " + literalSym->literal +getTerm(i,literalSym->suffix,0)+" | " + literalSym->literal + getTerm(i,literalSym->suffix,1)+" ))." << endl;
        ofInfoStream << endl;
    }
}

double timeCount(clock_t initTime){
    clock_t endNoPair = clock();
    double time_sNoPair = (double) (endNoPair - initTime) / (double) CLOCKS_PER_SEC;
    return time_sNoPair;
}

int main(int argc, char** argv) {
    
    bool debugStatExec = false;
    bool convertTptpFormat = false;
    StrategyParam::start = clock();
    FileOp::getTopPath(false);  
    vector<string> fileName;
    fileName.reserve(8);
//#define newTriDebug
#ifndef newTriDebug
    string filePath(argv[1]);
    string theoremFormat(argv[2]);
    string strategeMode(argv[3]);
    string runMode(argv[4]);
    string orCompetitionVersion(argv[5]);
    string workspace(argv[6]);
    string proDiv(argv[7]);
    FileOp::appPath = workspace;
#else
    string filePath = "";
    string theoremFormat = "";
    string strategeMode = "";
    string runMode = "";
    string orCompetitionVersion = "";
    string workspace = ""; 
    string proDiv = "";
#endif    
    if(filePath == "")
        filePath = FileOp::homePath + "/Desktop/2018CADE_FOF/KRS181+1.p";
    if(theoremFormat != "0" && theoremFormat != "1")
        theoremFormat = "0"; 
    if(strategeMode != "0" && strategeMode != "1" && strategeMode != "2" && strategeMode != "3" && strategeMode != "4")
        strategeMode = "1";
    if(runMode != "0" && runMode != "1" && runMode != "2" && runMode != "3" && runMode != "4" && runMode != "5" && runMode != "6" && runMode != "7" && runMode != "8" && runMode != "9" && runMode != "10" && runMode != "11")
        runMode = "4"; 
    if(orCompetitionVersion != "0" && orCompetitionVersion != "1")
        StrategyParam::orForCompition = true;
    else if(orCompetitionVersion == "0"){
        StrategyParam::orForCompition = false;
    }
    else if(orCompetitionVersion == "1"){
        StrategyParam::orForCompition = true;
    }
    
    if(StrategyParam::orForCompition){
        StrategyParam::problemDiv = 0;
    }
    else{
        if (proDiv == "0") {
            StrategyParam::problemDiv = 0;
        } else if (proDiv == "1") {
            StrategyParam::problemDiv = 1;
        } else {
            StrategyParam::problemDiv = 0;
        }
    }
    if(workspace == ""){
        if(StrategyParam::orForCompition) {
            workspace = "/home/ars01/Desktop/dist/output";
            FileOp::appPath = "/home/ars01/Desktop/dist";
        }
        else{
            workspace = FileOp::homePath  + "/Desktop/output";
            FileOp::appPath = "/home/ars01/Desktop/dist";
        }
    }
    else {
        if (StrategyParam::orForCompition)
            workspace += "/output";
        else
            workspace += "/Desktop/output";
    }
    bool workspaceExist = FileOp::isExistDir(workspace);
    string saveWorkPath = workspace + "/unsat";
    FileOp::isExistDir(saveWorkPath);
    FileOp::strategyMode = atoi(strategeMode.c_str());
    FileOp::runMode = atoi(runMode.c_str());
    double maxConvertCNFTime = 300;
    double maxConvertShortTime = 300;
    if(StrategyParam::orForCompition) {
        if ((FileOp::runMode != 0)&&(FileOp::runMode != 1)) {
            string usedTime(argv[7]);
            string timeForce(argv[8]);
            StrategyParam::timeForceLimit = stod(timeForce);
            StrategyParam::firstRunTime = stod(usedTime);
            maxConvertCNFTime = stod(timeForce);
            maxConvertShortTime = stod(timeForce);
            string inferRule(argv[9]);
            StrategyParam::cseRunMode = atoi(inferRule.c_str());
            string distriFirstTime(argv[10]);
            StrategyParam::distriTime = stod(distriFirstTime)/60;
            
        } else {
            string timeForce(argv[7]);
            StrategyParam::timeForceLimit = stod(timeForce);
            string inferRule(argv[8]);
            StrategyParam::cseRunMode = atoi(inferRule.c_str());
            string distriFirstTime(argv[9]);
            StrategyParam::distriTime = stod(distriFirstTime)/60;
        }
    }
    
    FileOp::workSpace = workspace;
    FileOp::getTopPath(StrategyParam::orForCompition);
    if(StrategyParam::orForCompition) {
        FileOp::CreateTotalOutputFile(true);
    }
    if((FileOp::runMode == 1)||(FileOp::runMode == 3)||(FileOp::runMode == 5)||(FileOp::runMode == 7)||(FileOp::runMode == 9)||(FileOp::runMode == 11)){
        FileOp::OperDir(filePath,&fileName);
    }
    else if((FileOp::runMode == 0)||(FileOp::runMode == 2)||(FileOp::runMode == 4)||(FileOp::runMode == 6)||(FileOp::runMode == 8)||(FileOp::runMode == 10)){
        fileName.push_back(filePath);
    }
    FileOp::clearRes();
    string logFile = "";
    if(theoremFormat == "0"){
        string final_res = FileOp::homePath + "/Desktop/output/tptp";
        if(StrategyParam::orForCompition)
            final_res = FileOp::workSpace + "/tptp";
        bool res = FileOp::isExistDir(final_res);
        if(res){
            FileOp::delAllFilesComplete(final_res);
        }
        logFile = FileOp::homePath + "/Desktop/output/tptp/log.txt";
        if(StrategyParam::orForCompition)
            logFile = FileOp::workSpace + "/tptp/log.txt";
    }
    for(auto& file:fileName)
    {
        if((FileOp::runMode == 1)||(FileOp::runMode == 3)||(FileOp::runMode == 5)||(FileOp::runMode == 7)||(FileOp::runMode == 9)||(FileOp::runMode == 11))
            StrategyParam::start = clock();
        if(theoremFormat == "0"){
            string pfilePath;
            if((FileOp::runMode == 1)||(FileOp::runMode == 3)||(FileOp::runMode == 5)||(FileOp::runMode == 7)||(FileOp::runMode == 9)||(FileOp::runMode == 11))
                pfilePath = filePath + "/" + file;
            else if((FileOp::runMode == 0)||(FileOp::runMode == 2)||(FileOp::runMode == 4)||(FileOp::runMode == 6)||(FileOp::runMode == 8)||(FileOp::runMode == 10))
                pfilePath = file;
            FileOp::judgeOriginalFileFullName = pfilePath;
            size_t pos = pfilePath.find_last_of('/');
            string fileName = pfilePath.substr(pos + 1);
            size_t posEnd = fileName.find_last_of('.');
            string fileNameNoSuffix = fileName.substr(0,posEnd);
            FileOp::judgeOriginalFileName = fileNameNoSuffix;
            if(StrategyParam::orForCompition){
               if((FileOp::runMode == 0)||(FileOp::runMode == 1)){
                   cout<<"start to proof:"<<fileNameNoSuffix<<endl;
               }
            }
            else
                cout<<"格式处理:"<<fileNameNoSuffix<<endl;
            string suffix=fileName.substr(posEnd+1);
            if(suffix == "p" || suffix == "P"|| suffix == "tptp"|| suffix == "TPTP"){
                string outFile;
                if (suffix == "p" || suffix == "P") {
                    if(debugStatExec){
                        cout << "start to use E for fof->cnf " << endl;
                    }
                    string cmd = "timeout 300 " + FileOp::homePath + "/Desktop/dist/eprover " + " --tstp-format " + " --cnf "+ pfilePath + "  >> " + FileOp::homePath + "/Desktop/output/" + fileNameNoSuffix +".p";
                    if(StrategyParam::orForCompition)
                        cmd = "timeout "+ to_string(maxConvertCNFTime) + " "+ FileOp::appPath + "/eprover " + " --tstp-format " + " --cnf "+ pfilePath + "  >> " + FileOp::workSpace + "/" + fileNameNoSuffix + ".p";
                    outFile = FileOp::homePath + "/Desktop/output/" + fileNameNoSuffix + ".p";
                    if(StrategyParam::orForCompition)
                        outFile = FileOp::workSpace + "/" + fileNameNoSuffix + ".p";                                                          
                    int res4 = system(const_cast<char*> (cmd.c_str()));                   
                    if(debugStatExec){
                        cout << cmd << endl;
                        string outCnfFormat = "cat " + outFile;
                        int resCnf = system(const_cast<char*> (outCnfFormat.c_str()));
                    }                    
                }
                else 
                    outFile = pfilePath;
                if(FileOp::isFileEmpty(outFile)){
                    if(StrategyParam::orForCompition) {
                        if ((FileOp::runMode == 0) || (FileOp::runMode == 1)) {
                            string strProofComp = FileOp::appPath + "/proofStatistics.txt";
                            FileOp::ofProofComp = new ofstream(strProofComp.c_str(), ios::app);
                            *FileOp::ofProofComp << fileNameNoSuffix << ":UNK‑Non" << endl;
                            FileOp::ofProofComp->close();
                        }
                    }
                    if (StrategyParam::orForCompition) {
                        cout << "UNK‑Non" << endl;
                    } 
                    else
                        cout<<"转换失败:"<<fileNameNoSuffix<<endl;
                    continue;
                }
                string strLine;
                string outFileTptp = FileOp::homePath + "/Desktop/output/"  + fileNameNoSuffix + ".p";
                if(StrategyParam::orForCompition)
                    outFileTptp = FileOp::workSpace + "/" + fileNameNoSuffix + ".p";//.tptp
                string final_Path = FileOp::homePath + "/Desktop/output/tptp";
                if(StrategyParam::orForCompition)
                    final_Path = FileOp::workSpace + "/tptp"; 
                maxConvertShortTime = StrategyParam::timeForceLimit - timeCount(StrategyParam::start);
                string cmdRes = "timeout 300 " + FileOp::homePath +"/Desktop/dist/tptp4X -tshorten -d" + final_Path + " " + outFileTptp + " >> " + logFile;                 
                if (debugStatExec) {
                    cout << "start to use TPTP4X for shorten " << endl;
                }  
                if(StrategyParam::orForCompition)
                    cmdRes = "timeout "+ to_string(maxConvertShortTime) + " "+ FileOp::appPath +"/tptp4X -tshorten -d" + final_Path + " " + outFileTptp + " >> " + logFile; 
                int res5 = system(const_cast<char*> (cmdRes.c_str()));
                remove(outFileTptp.c_str());
                string tptpShortFile = final_Path + "/" + fileNameNoSuffix + ".tptp";               
                if (debugStatExec) {
                    cout << cmdRes << endl;
                    string outCnfFormat = "cat " + tptpShortFile;
                    int resCnf = system(const_cast<char*> (outCnfFormat.c_str()));
                }
                if(access(tptpShortFile.c_str(),F_OK) == 0){
                    if (FileOp::isFileEmpty(tptpShortFile)) {
                        if (StrategyParam::orForCompition) {
                            if ((FileOp::runMode == 0) || (FileOp::runMode == 1)) {
                                string strProofComp = FileOp::appPath + "/proofStatistics.txt";
                                FileOp::ofProofComp = new ofstream(strProofComp.c_str(), ios::app);
                                *FileOp::ofProofComp << fileNameNoSuffix << ":UNK‑Non" << endl;
                                FileOp::ofProofComp->close();
                            }
                        }
                        if (StrategyParam::orForCompition) {
                            cout << "UNK‑Non" << endl;
                        }
                        else
                            cout << "转换失败:" << fileNameNoSuffix << endl;
                        continue;
                    }
                }
                else{
                    if(StrategyParam::orForCompition) {
                        if ((FileOp::runMode == 0) || (FileOp::runMode == 1)) {
                            string strProofComp = FileOp::appPath + "/proofStatistics.txt";
                            FileOp::ofProofComp = new ofstream(strProofComp.c_str(), ios::app);
                            *FileOp::ofProofComp << fileNameNoSuffix << ":UNK‑Non" << endl;
                            FileOp::ofProofComp->close();
                        }
                    }
                    if (StrategyParam::orForCompition) {
                        cout << "UNK‑Non" << endl;
                    } 
                    else
                        cout<<"转换失败:"<<fileNameNoSuffix<<endl;
                    continue;
                }
                bool addEqualityFlag = false;
                ifstream ifFolFileTptp(tptpShortFile);
                while(getline(ifFolFileTptp, strLine)) {
                    TrimStr(strLine); 
                    if(strLine.find("=",0)!= -1){
                       addEqualityFlag = true;
                       break;
                    }       
                }
                ifFolFileTptp.close();
                if(!addEqualityFlag){
                    string nameLast = final_Path + "/" + fileNameNoSuffix + "+short+noeq.tptp";
                    string changeFileNameCmd = "mv " + tptpShortFile + " " + nameLast;
                    int res6 = system(const_cast<char*> (changeFileNameCmd.c_str()));
                    file = nameLast;
                    if(convertTptpFormat){
                        cout<<"convertSuccess:"<<fileNameNoSuffix<<endl;
                        continue;
                    }
                    FileOp::AnalysisFileName(file);
                }
                else{
                    vector<funcAdd*> funcAddEquality;
                    funcAddEquality.reserve(16);
                    vector<literalAdd*> LiteralAddEquality;
                    LiteralAddEquality.reserve(16);
                    ifstream ifFolFileShort(tptpShortFile);
                    while(getline(ifFolFileShort, strLine)) {
                        TrimStr(strLine); 
                        if(strLine.length() == 0 || strLine[0] == '%'|| strLine[0] == 'c') 
                            continue;
                        char* ch = const_cast<char*> (strLine.c_str()); 
                        findEquality(ch,funcAddEquality,LiteralAddEquality);
                    }
                    ifFolFileShort.close();
                    if(funcAddEquality.size() != 0){
                        for (size_t rInd = 0; rInd < funcAddEquality.size() - 1; ++rInd) {
                            funcAdd* rA = funcAddEquality[rInd];
                            for (size_t rIndB = rInd + 1; rIndB < funcAddEquality.size();) {
                                funcAdd* rB = funcAddEquality[rIndB];
                                if ((rA->func == rB ->func) && (rA->suffix == rB->suffix)) {
                                    swap(funcAddEquality[rIndB], funcAddEquality[funcAddEquality.size() - 1]);
                                    funcAddEquality.pop_back();
                                } else
                                    ++rIndB;
                            }
                        }
                    }
                    if(LiteralAddEquality.size() != 0){
                        for (size_t rInd = 0; rInd < LiteralAddEquality.size() - 1; ++rInd) {
                            literalAdd* rA = LiteralAddEquality[rInd];
                            for (size_t rIndB = rInd + 1; rIndB < LiteralAddEquality.size();) {
                                literalAdd* rB = LiteralAddEquality[rIndB];
                                if ((rA->literal == rB ->literal ) && (rA->suffix == rB->suffix)) {
                                    swap(LiteralAddEquality[rIndB], LiteralAddEquality[LiteralAddEquality.size() - 1]);
                                    LiteralAddEquality.pop_back();
                                } else
                                    ++rIndB;
                            }
                        }
                    }
                    string nameLast = final_Path + "/" + fileNameNoSuffix + "+short+eq_rstfp.tptp";
                    ifstream ifFolFileOut(tptpShortFile);
                    ofstream ofInfoStreamOut(nameLast, ios::app);
                    bool addE = true;
                    while(getline(ifFolFileOut, strLine)) {
                        if(strLine[0] == '%' || strLine[0] == '-'){
                            ofInfoStreamOut << strLine << endl;
                            continue;
                        }
                        if(addE){
                            ofInfoStreamOut << "cnf(reflexivity,axiom," << endl;
                            ofInfoStreamOut << "    ( A = A ))." << endl;
                            ofInfoStreamOut << endl;
                            ofInfoStreamOut << "cnf(symmetry,axiom," << endl;
                            ofInfoStreamOut << "    ( A != B | B = A ))." << endl;
                            ofInfoStreamOut << endl;
                            ofInfoStreamOut << "cnf(transitivity,axiom," << endl;
                            ofInfoStreamOut << "    ( A != B | B != C | A = C ))." << endl;
                            ofInfoStreamOut << endl;
                            if(funcAddEquality.size() != 0){
                                for(int i=0; i<funcAddEquality.size(); ++i){
                                    if(funcAddEquality[i]->suffix >= 23){
                                        continue;
                                    }
                                    addFunc(ofInfoStreamOut,funcAddEquality[i]);
                                }
                            }
                            if(LiteralAddEquality.size() != 0){
                                for(int i=0; i<LiteralAddEquality.size(); ++i){
                                    if(LiteralAddEquality[i]->suffix >= 23){
                                        continue;
                                    }
                                    addLiteral(ofInfoStreamOut,LiteralAddEquality[i]);
                                }
                            }
                            addE = false;
                        }
                        ofInfoStreamOut << strLine << endl;                       
                    }
                    ifFolFileOut.close();
                    ofInfoStreamOut.close();
                    funcAddEquality.clear();
                    vector<funcAdd*>().swap(funcAddEquality);
                    LiteralAddEquality.clear();
                    vector<literalAdd*>().swap(LiteralAddEquality);
                    remove(tptpShortFile.c_str());
                    if(convertTptpFormat){
                        cout<<"convertSuccess:"<<fileNameNoSuffix<<endl;
                        continue;
                    }
                    file = nameLast;
                    FileOp::AnalysisFileName(file);
                                                          
                }    
            }
        }
        else{
            if ((FileOp::runMode == 1)||(FileOp::runMode == 3)||(FileOp::runMode == 5)||(FileOp::runMode == 7)||(FileOp::runMode == 9)||(FileOp::runMode == 11)) {
                FileOp::AnalysisFileName(filePath + "/" + file);
                if(StrategyParam::orForCompition){
                    FileOp::judgeOriginalFileFullName = filePath + "/" + file;
                    size_t pos = FileOp::judgeOriginalFileFullName.find_last_of('/');
                    string fileName = FileOp::judgeOriginalFileFullName.substr(pos + 1);
                    size_t posEnd = fileName.find_last_of('.');
                    string fileNameNoSuffix = fileName.substr(0, posEnd);
                    FileOp::judgeOriginalFileName = fileNameNoSuffix;
                }
            } else if ((FileOp::runMode == 0)||(FileOp::runMode == 2)||(FileOp::runMode == 4)||(FileOp::runMode == 6)||(FileOp::runMode == 8)||(FileOp::runMode == 10)) {
                FileOp::AnalysisFileName(file);
                if(StrategyParam::orForCompition){
                    FileOp::judgeOriginalFileFullName = file;
                    size_t pos = file.find_last_of('/');
                    string fileName = file.substr(pos + 1);
                    size_t posEnd = fileName.find_last_of('.');
                    string fileNameNoSuffix = fileName.substr(0, posEnd);
                    FileOp::judgeOriginalFileName = fileNameNoSuffix;
                }
            }
        }
#define initPath
#ifndef initPath
            FileOp::AnalysisFileName(filePath + "/" + file);
#else
            //FileOp::AnalysisFileName(file);
#endif
        if(convertTptpFormat) {
            continue;
        }
        if (1) {           
            pid_t pid;
            pid = fork();
            if(pid == 0)
            {
                Formula formual;
                INT32 startId = InitFunction::StrToAny("-1");
                if (startId == -1)
                    cprover::PreProcess(formual);
                string sPreprocess = formual.GetFormulaInfo();
                sPreprocess += "\n" + formual.ToString(false);
                FileOp::WritePreFile(sPreprocess);
                if ((StrategyParam::orForCompition) && (FileOp::runMode != 0) && (FileOp::runMode != 1)) {
                    string str = "";
                    str += formual.competInitClauseSet();
                    FileOp::prinfPreResForE(str); 
                }
                formual.DelSet();
                exit(0);
            }
            else if(pid >0)
            {
                waitpid(pid,NULL,0);
            }    
        }
        if (FileOp::strategyMode == 3) {
            FileOp::strategyFileSize = FileOp::strategyFileSizeFunc(FileOp::strategyFile);
            for(int id = 0; id < FileOp::strategyFileSize; ++id){
                cout<< "the " + to_string(id+1) + "th; " + " total numbers:" + to_string(FileOp::strategyFileSize) << endl;               
                StrategyParam::start = clock();
                pid_t child;
                child = fork();
                if (child == 0) {
                    RESULT s;
                    s = Prover::proverTestTptp(id);
                    exit(0);
                } 
                else if (child > 0) {
                    waitpid(child, NULL, 0);
                    
                }               
                if (FileOp::isExistProof(FileOp::g_OutputPath.c_str())) {
                    string strategy = FileOp::getStrategyNTptp(id);
                    FileOp::copyAndDelOrgFile(strategy,theoremFormat);
                    break;
                }
            }
        }
        else if(FileOp::strategyMode == 4){
            string idString(argv[8]);
            int id = atoi(idString.c_str());
            string totalString(argv[9]);
            int totalNum = atoi(totalString.c_str());
            cout<< "the " + to_string(id+1) + "th; " + " total numbers:" + to_string(totalNum) << endl;
            StrategyParam::start = clock();
            pid_t child;
            child = fork();
            if (child == 0) {
                RESULT s;
                s = Prover::proverTestTptp(id);
                exit(0);
            }
            else if (child > 0) {
                waitpid(child, NULL, 0);
            }      
        }
        else {
            if(StrategyParam::orForCompition){
                if (debugStatExec) {
                    cout << "start to run CSE for proof search " << endl;
                }
                RESULT s;
                s = Prover::prover();
            }
            else {
                pid_t child;
                child = fork();
                if (child == 0) {

                    if (debugStatExec) {
                        cout << "start to run CSE for proof search " << endl;
                    }                   
                    RESULT s;
                    s = Prover::prover();
                    exit(0);
                } 
                else if (child > 0) {
                    waitpid(child, NULL, 0);
                }
            }
        }      
    }
    fileName.clear();
    vector<string>().swap(fileName); 
    int res[3];
    FileOp::getRes(res);
    return 0;
}

