/*
*  Copyright 2020 by the author. It is protected by applicable copyright laws. This program does not
*  give you or anybody else a right or licence to distribute, modify, copy, compile, create derivatives.
*/

#ifndef SUBSUMPTION_H
#define SUBSUMPTION_H
#include "Unify.h"
#include "Term.h"
#include "Clause.h"
#include "Literal.h"
#include "globalFunc.h"
#include "dataTypeDef.h"
class Subsumption {
public:
    Subsumption();
    Subsumption(const Subsumption& orig);
    virtual ~Subsumption();
    static bool CheckTautology(vector<string>& vectLit);
    bool GenCla(Clause* queryCla, UINT32& uClaCode) {
        set<INT32> vFindedClaID; 
        vector<UINT32> vInvaildClaId;
        for (UINT16 ind = 0; ind < queryCla->uLitNum; ++ind) {
            Literal*qLit = queryCla->LitPtr[ind]; 
            vector<Literal*>*vCandLits = qLit->getPredLst(); 
            for (auto& candLitPtr : *vCandLits)
            {
                Clause* candCla = candLitPtr->claPtr; 
                INT32 candRow = candCla->uClaId; 
                if (candRow == queryCla->uClaId) continue; 
                if (candCla->uLitNum > queryCla->uLitNum) continue; 
                if (vFindedClaID.find(candRow) != vFindedClaID.end())
                    continue; 
                vFindedClaID.insert(candRow); 
                if (ConClaIncludeVarCla(queryCla, candCla, vInvaildClaId))
                {
                    uClaCode = candRow;
                    return true;
                }
            }
        }
        return false;
    }

    bool GenCla(Formula& fol, Clause* queryCla, vector<UINT32>&vInvaildClaId, string& rundacyMsg) {
        set<INT32> vFindedClaID; 
        if(queryCla->uLitNum >= 50){
            rundacyMsg = "|[无效]文字数超过限定大小;";
            return true;
        }
        for (UINT16 ind = 0; ind < queryCla->uLitNum; ++ind) {
            Literal*qLit = queryCla->LitPtr[ind];
            vector<Literal*>*vCandLits = qLit->getPredLst();
            vInvaildClaId.reserve(8);
            rundacyMsg = "|[Y]";
            for (auto& candLitPtr : *vCandLits) {
                Clause* candCla = candLitPtr->claPtr;
                INT32 candRow = candCla->uClaId;
                if (candRow == queryCla->uClaId) continue;
                if (candCla->uLitNum > queryCla->uLitNum) continue;
                if (vFindedClaID.find(candRow) != vFindedClaID.end())
                    continue;
                vFindedClaID.insert(candRow);
                if (ConClaIncludeVarCla(queryCla, candCla, vInvaildClaId)) {
                    rundacyMsg = "|[无效]包含C_" + to_string(candRow) + ":导致冗余的文字";
                    for (auto&id : vInvaildClaId) {
                        rundacyMsg += to_string(GetClaId(id)) + "_" + to_string(GetVarId(id)) + ";";
                        addWeight(fol,id);
                    }
                    return true;
                }
            }
        }
        return false;
    }
    void addWeight(Formula& fol,UINT32 id) {
        if (StrategyParam::useSaturate) {
            for (auto& aClp : fol.setDeductCla) {
                if (aClp->uClaId == id) {
                    if (aClp->uLitNum == 1)
                        aClp->adeductNum += 2;
                    else
                        aClp->reduntTimes += 50;
                }
            }
        } else {
            for (auto& aClp : fol.setDeductCla) {
                if (aClp->uClaId == id) {
                    if (aClp->uLitNum == 1)
                        aClp->adeductNum += 2;
                    else
                        aClp->reduntTimes += 50;
                }
            }
        }
    }

    void InstCla(Clause&queryCla, set<Clause*>&vRundacyClas) {
        for (UINT16 i = 0; i < queryCla.uLitNum; ++i) {
            for (auto&candLitP : *queryCla.LitPtr[i]->getPredLst()) {
                Clause* conCla = candLitP->claPtr;
                if (conCla == &queryCla || conCla->uLitNum < queryCla.uLitNum || vRundacyClas.find(conCla) != vRundacyClas.end())
                    continue;
                if (ConClaIncludeVarCla(conCla, &queryCla))
                {
                    vRundacyClas.insert(conCla);
                }
            }
        }
    }

    void InstClaDirect(Clause&queryCla,Clause& cla_Del, set<Clause*>&vRundacyClas) {
        for (UINT16 i = 0; i < queryCla.uLitNum; ++i) {
            Clause* conCla = &cla_Del;
            if (conCla == &queryCla || conCla->uLitNum < queryCla.uLitNum || vRundacyClas.find(conCla) != vRundacyClas.end())
                continue;
            if (ConClaIncludeVarCla(conCla, &queryCla)) {
                vRundacyClas.insert(conCla);
            }
        }
    }
    
    bool ConClaIncludeVarCla(Clause* conCla, Clause* varCla, vector<UINT32>& vInvaildClaId) {      
        vector<vector < UINT32>> staVarChgTs;
        vector<UINT32> staLitsPairId;
        UINT16 varLitInd = 0, conLitInd = 0;
        Unify unify;
        Term** varBind = nullptr;
        if (varCla->MaxVarId() > 0)
        {
            varBind = new Term*[varCla->MaxVarId()]; 	
            memset(varBind, 0, sizeof (Term*)*(varCla->MaxVarId())); 
        }
        while (1) {
            bool isFind = false;
            while (conLitInd != conCla->uLitNum)
            {
                isFind = false;
                vector<UINT32>chgVarT;
                if (unify.Match(conCla->LitPtr[conLitInd], varCla->LitPtr[varLitInd], varBind, chgVarT))
                {

                    isFind = true;
                    staLitsPairId.push_back(varLitInd); 
                    staLitsPairId.push_back(conLitInd); 
                    staVarChgTs.push_back(chgVarT); 
                    vInvaildClaId.push_back(conCla->LitPtr[conLitInd]->ParentLitCode()); 	
                    if (++varLitInd == varCla->uLitNum)
                        break;
                    conLitInd = 0;   
                }                   
                else {
                    if (!chgVarT.empty()) {
                        for (auto&ind : chgVarT)
                            varBind[ind] = nullptr;
                    }
                    ++conLitInd;
                }
            }
            if (isFind) {
                if (!staVarChgTs.empty()) {
                    for (auto& vt : staVarChgTs) {
                        for (auto& ind : vt) {
                            varBind[ind] = nullptr;
                        }
                    }
                    staVarChgTs.clear();
                }
                if (varBind)
                    delete[]varBind;
                return true;
            }   
            if (staVarChgTs.empty()) {
                if (varBind)
                    delete[]varBind;
                return false;
            }
            conLitInd = staLitsPairId.back();
            staLitsPairId.pop_back();
            ++conLitInd;
            varLitInd = staLitsPairId.back();
            staLitsPairId.pop_back();
            vector<UINT32 >&changeVarTMP = staVarChgTs.back();
            if (!changeVarTMP.empty()) {
                for (auto&ind : changeVarTMP) {
                    varBind[ind] = nullptr;
                }
            }
            staVarChgTs.pop_back();
            vInvaildClaId.pop_back();
        }
    }

    bool ConClaIncludeVarCla(Clause* conCla, Clause* varCla) {
        vector<vector < UINT32>> staVarChgTs;
        vector<UINT32> staLitsPairId;
        UINT16 varLitInd = 0, conLitInd = 0;
        Unify unify;
        Term** varBind = nullptr;
        if (varCla->MaxVarId() > 0) {
            varBind = new Term*[varCla->MaxVarId()]; 	
            memset(varBind, 0, sizeof (Term*)*(varCla->MaxVarId())); 
        }
        while (1) {
            bool isFind = false;
            while (conLitInd != conCla->uLitNum) {
                isFind = false;
                vector<UINT32>chgVarT;
                if (unify.Match(conCla->LitPtr[conLitInd], varCla->LitPtr[varLitInd], varBind, chgVarT)) {                   
                    isFind = true;
                    staLitsPairId.push_back(varLitInd);
                    staLitsPairId.push_back(conLitInd);
                    staVarChgTs.push_back(chgVarT);	
                    if (++varLitInd == varCla->uLitNum)
                        break;
                    conLitInd = 0; 
                }                    
                else {
                    if (!chgVarT.empty()) {
                        for (auto&ind : chgVarT)
                            varBind[ind] = nullptr;
                    }
                    ++conLitInd;
                }
            }
            if (isFind) {
                if (!staVarChgTs.empty()) {
                    for (auto&vt : staVarChgTs) {
                        for (auto&ind : vt)
                            varBind[ind] = nullptr;
                        vector<UINT32>().swap(vt);
                    }
                }
                if (varBind) delete[]varBind;
                return true;
            } 
            if (staVarChgTs.empty()) {
                if (varBind) delete[]varBind;
                return false;
            }
            ++conLitInd;
            vector<UINT32 >&changeVarTMP = staVarChgTs.back();
            if (!changeVarTMP.empty()) {
                for (auto&ind : changeVarTMP)
                    varBind[ind] = nullptr;
            }
            staVarChgTs.pop_back();
        }
    }
private:

};

#endif /* SUBSUMPTION_H */

